"""
Domain extensions for curate-ipsum.

This module provides BRS domain extensions specific to code mutation testing.
"""

from curate_ipsum.domains.code_mutation_smoke import DOMAIN_ID, SMOKE_LEVELS

__all__ = ["DOMAIN_ID", "SMOKE_LEVELS"]
